IF OBJECT_ID('dbo.DBA_PM_BlockAndWaitStats') IS NULL
BEGIN
	CREATE TABLE dbo.DBA_PM_BlockAndWaitStats 
	(ObjId			bigint identity(1,1) not null			--key
	,CurrTime		datetime		NOT NULL				--time of the block/wait.
	,spid			smallint		NOT NULL				--SPID		
	,BlockingSpid	smallint		NOT NULL				--SPID of the session that is blocking the request.  0 is not blocked.  -2 is orphaned
																--dist transaction, -4 is undetermined due to state transitions 
	,IsHeadBlocker	bit				NOT NULL				--finds any blocking tasks that are not themselves blocked.  
	,ServerName		varchar(40)		NOT NULL				--Server/InstanceName
	,DBName			varchar(50)		NULL					--database against which the request arrived		
	,waittime		bigint			NULL				--total wait time for this wait type, in ms, inclusive of signal_wait_time. 
	,wait_type		nvarchar(60)	NULL					--name of the wait type, ie LCK_M_X, WRITELOG, NETWORKIO,etc
	,wait_resource	nvarchar(512)	NULL					--if the request is blocked then what is it waiting on, ie db:file:objectid
	,resource_description nvarchar(1024) null				--what are we consuming at the task level. this holds a lot of information when it can.
	--,LockResourceID			bigint null						--the "object" that is locked that we are waiting on.  Likely an objectid, but could be partitionid or HoBTid (heap or B-tree)
	--,LockResourceType		varchar(100) null				--RID, EXTENT, HOBT, FILE, PAGE, etc
	--,LockRequestMode		varchar(100) null				--either the granted or requested mode, ie IX, S, I
	,status			nvarchar(30)	NOT NULL				--status of the request (running, runnable, sleeping, suspended, etc)
	,CommandType	nchar(32)		NOT NULL				--type of command (DBCC, SELECT, INSERT, BACKUP LOG, etc)
	,cpu_ms			int				NULL					--CPU used (in ms) at the request level
	--,reads			bigint			NOT NULL				--reads at the request level
	--,writes			bigint			NOT NULL				--writes at the request level
	,physical_io		bigint			NULL
	--,ElapsedMS		decimal(15,3)	NULL					--total time since request arrival
	--,StatementText	varchar(2000)	NULL					--actual statement within the batch that is executing.  Is not guaranteed to always be accurate
	,SQLText		nvarchar(MAX)	NULL					--text of the request
	,program_name	nvarchar(256)	NOT NULL				--from what application
	,hostname		nvarchar(256)	NULL					--who made the call
	,login_name		nvarchar(128)	NOT NULL				--SQL Server login name under which the session is currently executing. 
	,last_batch		datetime		NOT NULL				--Timestamp when the request arrived.
	,open_tran		smallint		NOT NULL				--number of open transactions for the request
	,tran_iso_level	varchar(40)		NULL					--transaction isolation level of the request
	,statement_start_offset  int		NULL					--used to calculate StatementText in the API view
	,statement_end_offset int		NULL
	)
END
ELSE
BEGIN
	--these cols are too expensive to retrieve
	IF EXISTS (select * from sys.columns WHERE object_id = object_id('dbo.DBA_PM_BlockAndWaitStats') AND name = 'LockResourceID')
	BEGIN
		PRINT 'Dropping deprecated cols'
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats DROP COLUMN LockResourceID');
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats DROP COLUMN LockResourceType');
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats DROP COLUMN LockRequestMode');
	END
	IF EXISTS (select * from sys.columns WHERE object_id = object_id('dbo.DBA_PM_BlockAndWaitStats') AND name = 'reads')
	BEGIN
		PRINT 'Dropping deprecated cols'
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats DROP COLUMN reads');
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats DROP COLUMN writes');
	END
	IF NOT EXISTS (select * from sys.columns WHERE object_id = object_id('dbo.DBA_PM_BlockAndWaitStats') AND name = 'physical_io')
	BEGIN
		PRINT 'Adding physical_io'
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats ADD physical_io bigint');
	END
	IF EXISTS (select * from sys.columns WHERE object_id = object_id('dbo.DBA_PM_BlockAndWaitStats') AND name = 'StatementText')
	BEGIN
		PRINT 'Adding/removing StatementText cols'
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats DROP COLUMN StatementText');
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats ADD statement_start_offset int NULL');
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats ADD statement_end_offset bigint NULL');
	END
	IF EXISTS (select * from sys.columns WHERE object_id = object_id('dbo.DBA_PM_BlockAndWaitStats') AND name = 'SQLText' AND max_length = 2000)
	BEGIN
		PRINT 'Altering SQLText'
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats ALTER COLUMN SQLText nvarchar(max)');
	END
	IF EXISTS (select * from sys.columns WHERE object_id = object_id('dbo.DBA_PM_BlockAndWaitStats') AND name = 'ElapsedMS')
	BEGIN
		PRINT 'Removing ElapsedMS'
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats DROP COLUMN ElapsedMS');
	END
	IF EXISTS (select * from sys.columns WHERE object_id = object_id('dbo.DBA_PM_BlockAndWaitStats') AND name = 'tran_iso_level' AND max_length <> 40)
	BEGIN
		PRINT 'Altering SQLText'
		EXEC ('ALTER TABLE dbo.DBA_PM_BlockAndWaitStats ALTER COLUMN tran_iso_level varchar(40)');
	END
END
GO
