/*
	--Creates SMSPHsfu if it does not exist.  
	--Attempts to create the db on R and L per the SFDA standard drive layout.  
	--If it can't then it creates it on the same drive/folder structure as the first db found named like %tnd%
	--You can always build the SMSPHsfu yourself, in which case this script does nothing.  

*/
SET NOCOUNT ON 

IF NOT EXISTS (SELECT 1 FROM master..sysdatabases WHERE NAME = 'SMSPHsfu')
BEGIN
	PRINT 'Database is not found.  Database will be created.'	
	
	-------------------------------------------------------------
	-- Determine file names 
	-------------------------------------------------------------
	DECLARE @vcDataDir VARCHAR(300) 
	DECLARE @vcDataDirFile VARCHAR(300) 
	DECLARE @vcLogDirFile VARCHAR(300)  
	DECLARE @SQL varchar(max) 
	DECLARE @InstanceName varchar(200)
	SELECT @InstanceName = COALESCE(CONVERT(varchar(200),SERVERPROPERTY('instancename')),'') 
	CREATE TABLE #File_Results (
		File_Exists int,
		File_is_a_Directory int,
		Parent_Directory_Exists int
	)
	
	--does R:\Data_dev exist?  
	INSERT INTO #File_Results EXEC xp_fileexist 'R:\data_dev'
	IF EXISTS (SELECT * FROM #File_Results WHERE File_is_a_Directory = 1)
	BEGIN
		SELECT @vcDataDirFile = 'R:\Data_dev\SMSPHsfu' + CASE WHEN @InstanceName = '' THEN '' ELSE '_' + @InstanceName END + '.mdf'
	END
	DELETE FROM #File_Results
	
	--does L:\Log_dev exist?  
	INSERT INTO #File_Results EXEC xp_fileexist 'L:\Log_dev'
	IF EXISTS (SELECT * FROM #File_Results WHERE File_is_a_Directory = 1)
	BEGIN
		SELECT @vcLogDirFile = 'L:\Log_dev\SMSPHsfu' + CASE WHEN @InstanceName = '' THEN '' ELSE '_' + @InstanceName END + '.ldf'
	END
	DELETE FROM #File_Results
	
	--get the mdf file for the first tnd
	--get just the path
	select @vcDataDir = filename  
	from master..sysdatabases 
	where name like '%tnd%'
	select @vcDataDir = substring(@vcDataDir,1,(LEN(@vcDataDir) - CHARINDEX('\',REVERSE(@vcDataDir))) + 1)
	
	IF @vcDataDirFile IS NULL
	BEGIN
		--then use TND's location
		SELECT @vcDataDirFile=@vcDataDir+'SMSPHsfu' + CASE WHEN @InstanceName = '' THEN '' ELSE '_' + @InstanceName END + '.mdf'
	END

	IF @vcLogDirFile IS NULL
	BEGIN
		--then use TND's location
		SELECT @vcLogDirFile=@vcDataDir+'SMSPHsfu' + CASE WHEN @InstanceName = '' THEN '' ELSE '_' + @InstanceName END + '.ldf'
	END

 
	-------------------------------------------------------------
	-- Build and execute the Create Database command
	-------------------------------------------------------------
	PRINT 'Executing the Create Database command...'
	SELECT @SQL = '
	CREATE DATABASE SMSPHsfu ON PRIMARY
	( NAME = N''SMSPHsfu'', FILENAME = N'''+@vcDataDirFile+''' , SIZE = 2048000KB, FILEGROWTH = 51200KB)
	LOG ON 
	( NAME = N''SMSPHsfu_log'', FILENAME = N'''+@vcLogDirFile+''' , SIZE = 204800KB , FILEGROWTH = 51200KB )
	'
	PRINT @SQL 
	BEGIN TRY
		EXEC(@SQL)
		EXEC ('ALTER DATABASE SMSPHsfu SET RECOVERY SIMPLE');
	END TRY
	BEGIN CATCH
		PRINT ERROR_MESSAGE()
		PRINT ERROR_NUMBER()
		PRINT 'The above error occurred will trying to CREATE DATABASE.  It is likely that the directory does not exist.'
		RAISERROR ('See previous errors',16,1) 
	END CATCH
	
END
ELSE
BEGIN
	PRINT 'Database SMSPHsfu already exists.  No Action taken'
END

GO 

