IF exists (select 1 from sysobjects where name='DBA_PM_Addin_AbandonedSpidKiller' and type='p')
   DROP procedure DBA_PM_Addin_AbandonedSpidKiller
GO

  
CREATE Procedure DBA_PM_Addin_AbandonedSpidKiller       
As    
BEGIN        
	set nocount on             
	
	--This job will monitor the system for a specific type of abandonded spid - a connection done with it's
	--task and sitting at AWAITING COMMAND but still holding an open transaction.  These connections must also be blocking 
	--at least one other connection, the original connection must be abandoned for at least 5 minutes, and the other connection 
	--waiting for at least 1 minute.  When such qualifying abandonded spids are detected their information is logged into 
	--SMSPHcdb and a KILL command created and executed to ter
	
	SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
	--get the most recent run from DBA_PM_WaitAndBlockMonitor
	DECLARE @CurrTime datetime, @spid int , @exec_str varchar(100)
	SELECT @CurrTime = MAX(CurrTime) FROM DBA_PM_BlockAndWaitStats WITH (NOLOCK)
	

		DECLARE Killer CURSOR FOR 
			SELECT spid 
			FROM DBA_PM_BlockAndWaitStats WITH (NOLOCK)
			WHERE CurrTime = @CurrTime
			AND tran_iso_level = 'Abandoned Blocking Spid for > 60 sec?'
			AND DATEDIFF(s,last_batch,CurrTime) > 300					--spid has to be out there for 5 mins.  
		OPEN Killer
		FETCH NEXT FROM Killer INTO @spid
		WHILE (@@FETCH_STATUS = 0)
		BEGIN
			BEGIN TRY
				SELECT @exec_str = 'Kill ' + convert(varchar(20),@spid)
				EXEC (@exec_str)
				INSERT INTO DBA_PM_SpidsKilled VALUES (@CurrTime,@spid,'Success')
			END TRY
			BEGIN CATCH
				INSERT INTO DBA_PM_SpidsKilled VALUES (@CurrTime,@spid,ERROR_MESSAGE())
			END CATCH
		
			FETCH NEXT FROM Killer INTO @spid
		END
		CLOSE Killer
		DEALLOCATE Killer	
		
END;
GO		