IF  EXISTS (SELECT * FROM sysobjects WHERE name = N'DBA_PM_Addin_CaptureDeadlocks' and xtype = 'p')
begin
  drop procedure DBA_PM_Addin_CaptureDeadlocks  
end
go

set QUOTED_IDENTIFIER on
go

DECLARE @SQL varchar(8000)


if dbo.DBA_PM_SQLVersion () = '2008' 
BEGIN
SELECT @SQL = 
'
create procedure DBA_PM_Addin_CaptureDeadlocks 
as
begin
  set nocount on
  set QUOTED_IDENTIFIER on
  declare @guid uniqueidentifier, @ThisSp varchar(128), @now datetime, @nowprev datetime,
  @alert_name varchar(128), @alert_yellow int, @alert_red int, @execution_target_database varchar(128)
  select @guid = newid(),@ThisSp = ''DBA_PM_Addin_CaptureDeadlocks'', @execution_target_database = ''*'', @now = GETDATE()

  Declare @retentionRows int, @TotalDeadlockRecordCount int, @recordcount int
  SELECT @nowprev = MAX(CurrTime) from dbo.DBA_PM_BlockAndWaitStats   

  --If the event session does not exist then create it.
  IF NOT EXISTS(SELECT * FROM sys.server_event_sessions WHERE name=''RingBufferDeadlockTarget'')
  BEGIN
    CREATE EVENT SESSION RingBufferDeadlockTarget  --DROP EVENT session RingBufferDeadlockTarget ON SERVER
    ON SERVER 
    ADD EVENT sqlserver.xml_deadlock_report    ,
    ADD EVENT sqlserver.lock_deadlock 
    ( 
      ACTION (sqlserver.sql_text, sqlserver.tsql_stack, sqlserver.username, 
	  sqlserver.session_id, sqlserver.request_id) --, sqlserver.xml_deadlock_report) 
      )
      ADD TARGET package0.ring_buffer    
          (SET max_memory = 4096) 
      WITH (max_dispatch_latency = 1 seconds)
  END

  --If the event session is not running then start it
  IF NOT EXISTS (SELECT 1 
    FROM sys.dm_xe_session_targets t INNER JOIN sys.dm_xe_sessions s
    ON t.event_session_address = s.address
    WHERE s.name = ''RingBufferDeadlockTarget'' AND t.target_name = ''ring_buffer''
  ) 
  begin
    ALTER EVENT SESSION RingBufferDeadlockTarget ON SERVER
    STATE = start
  end
  
  	if datediff (dd,COALESCE(@nowprev,''1/1/1980''),@now) > 0                      
	BEGIN    
	
  --Get the deadlocks into a temp table
  if OBJECT_ID(''tempdb..#tmp'') is not null 
    drop table #tmp
  create table #tmp (DeadlockGraph varchar(max))


  begin try
  insert into #tmp (DeadlockGraph)
  select 
  --CAST(     --This REPLACE call is to cover a bug in the trace XML from the event system.
			REPLACE(  --The bug is a mismatched begin/end tag pair.
				REPLACE(XEventData.XEvent.value(''(data/value)[1]'', ''varchar(max)''), 
				''<victim-list>'', ''<deadlock><victim-list>''),
			''<process-list>'',''</victim-list><process-list>'')
		--as xml) 
		as DeadlockGraph
  FROM
  (select CAST(target_data as xml) as TargetData
   from sys.dm_xe_session_targets st
   join sys.dm_xe_sessions s on s.address = st.event_session_address
   where name = ''RingBufferDeadlockTarget'' --system_health
  ) AS Data
  CROSS APPLY TargetData.nodes (''//RingBufferTarget/event'') AS XEventData (XEvent)
  where XEventData.XEvent.value(''@name'', ''varchar(4000)'') = ''xml_deadlock_report''
  end try
  begin catch
    --print ''in catch block''  --DBA_PM_CaptureDeadlocks  --select * from DBA_PM_DeadlockSnapshotCorruptedBuffer
    delete from #tmp
    ALTER EVENT SESSION RingBufferDeadlockTarget ON SERVER
    STATE = stop; 
    ALTER EVENT SESSION RingBufferDeadlockTarget ON SERVER
    STATE = start;
     insert into DBA_PM_DeadlockSnapshotCorruptedBuffer(currtime, Message, ServerName) values (GETDATE(), ''Read error encountered, probably XML truncation'' ,@@SERVERNAME);    
  end catch

  select @recordcount = COUNT(*) from #tmp 
  insert into DBA_PM_DeadlockSnapshot (DeadlockGraph, DeadlockGraphCheckSum, ServerName)
  select tmp.DeadlockGraph, 0, @@SERVERNAME from #tmp tmp
  --No longer needed since the buffer is cleared
  --left outer join DBA_PM_DeadlockSnapshot DBA 
  --on (tmp.DeadlockGraphCheckSum = dba.DeadlockGraphCheckSum and convert(varchar(max),tmp.DeadlockGraph) = convert(varchar(max),dba.DeadlockGraph ))
  --where DBA.DeadlockGraphCheckSum is null
  

  --Clear the event session
  ALTER EVENT SESSION RingBufferDeadlockTarget ON SERVER
  STATE = stop ;
  ALTER EVENT SESSION RingBufferDeadlockTarget ON SERVER
  STATE = start ;
  
	END;
end
'
EXEC (@SQL)
--PRINT @SQL
END
ELSE IF dbo.DBA_PM_SQLVersion () = '2005' 
BEGIN
SELECT @SQL = 
'
create procedure DBA_PM_Addin_CaptureDeadlocks 
as
begin
  set nocount on
  set QUOTED_IDENTIFIER on
  PRINT ''DBA_PM_Addin_CaptureDeadlocks not supported on SQL 2005''
end
'
EXEC (@SQL)
END
else
BEGIN
	RAISERROR ('SFU Tool not tested on anything but 2008 and 2005. ',16,1) WITH LOG;
END

GO