IF exists (select 1 from sysobjects where name='DBA_PM_Addin_IO' and type='p')
   DROP procedure DBA_PM_Addin_IO
GO

  
CREATE Procedure DBA_PM_Addin_IO       
As    
BEGIN        
	set nocount on             
	    
	DECLARE @Now DATETIME , @nowprev DATETIME     , @dbname varchar(200) 
	SELECT @nowprev = MAX(Currtime) from DBA_PM_IOStalls   
	SELECT @now = GETDATE()    
	
	DECLARE @PollingInterval int
	SELECT @PollingInterval = COALESCE(ConfigValueInt,7) 
	FROM DBA_PM_Config
	WHERE ConfigSetting = 'DBA_PM_Addin_IO'     
	
	--let's run this about every 7 hours.  
	if datediff (hh,COALESCE(@nowprev,'1/1/1980'),@now) > @PollingInterval   
	BEGIN	
		--Step 1:
		--io stalls 
		WITH DBIO AS
		(
			Select 
				db_name(ivfs.database_id) as db,
				case when mf.type = 1 THEN 'log' else 'data' end as file_type,
				sum(ivfs.num_of_bytes_read + ivfs.num_of_bytes_written) as io,
				sum(ivfs.io_stall) as io_stall
			from sys.dm_io_virtual_file_STATS(NULL,NULL) AS IVFS
			JOIN sys.master_files as mf
				ON ivfs.database_id = mf.database_id
				AND ivfs.file_id = mf.file_id
			GROUP BY db_name(ivfs.database_id), mf.type
		)
		, Totals AS
		(
			SELECT db,file_type,
			cast(1. * io/(1024*1024) as decimal(12,2)) as io_mb,
			cast(io_stall/1000. as decimal(12,2)) as io_stall_ms,
			cast(100. * io_stall/sum(io_stall) over() as decimal(10,2)) as io_stall_pct
			from dbio
		)
		INSERT INTO DBA_PM_IOStalls
		SELECT 
			GETDATE(),
			@@SERVERNAME,
			db,
			file_type,
			io_mb,
			io_stall_ms,
			io_stall_pct
		FROM Totals 
		WHERE io_stall_pct > 1
	
	

	
	END;   

END
GO  