IF exists (select 1 from sysobjects where name='DBA_PM_Addin_IndexMetrics' and type='p')
   DROP procedure DBA_PM_Addin_IndexMetrics
GO

  
CREATE Procedure DBA_PM_Addin_IndexMetrics       
As    
BEGIN        
	set nocount on             
	    
	DECLARE @Now DATETIME , @nowprev DATETIME     , @dbname varchar(200) 
	SELECT @nowprev = MAX(Currtime) from DBA_PM_IndexBlocking   
	SELECT @now = GETDATE()   
	
	DECLARE @PollingInterval int
	SELECT @PollingInterval = COALESCE(ConfigValueInt,4) 
	FROM DBA_PM_Config
	WHERE ConfigSetting = 'DBA_PM_Addin_IndexMetrics'     
	
	--let's run this about every 4 hours.  
	if datediff (hh,COALESCE(@nowprev,'1/1/1980'),@now) > @PollingInterval   
	BEGIN	
		--Step 1:
		--when a pagelock or rowlock is completed a row is logged to with the two cols below populated.  
		--this does not include range locking
		--Block_Waittime_ms is a good indicator of blocking in general, as long as the object is still cached.  
		--if an index is listed but it is not being used (query sys.dm_db_index_usage_stats) then perhaps the index can be dropped.  
		--perhaps also a better index should be defined
		--PRINT 'Starting Step1'
		INSERT INTO DBA_PM_IndexBlocking
		select 
			GETDATE() AS CurrTime,
			@@SERVERNAME AS ServerName,
			db_name(database_id) DB,
			QUOTENAME(OBJECT_SCHEMA_NAME(object_id, database_id)) + N'.' + QUOTENAME(OBJECT_NAME(object_id, database_id)) ObjDetails,
			row_lock_wait_in_ms + page_lock_wait_in_ms Block_Waittime_ms,
			index_id,
			partition_number
		from sys.dm_db_index_operational_stats(NULL,NULL,NULL,NULL)
		--WHERE database_id IN (select database_id FROM sys.databases WHERE name LIKE '%tnd%')
		WHERE (row_lock_wait_in_ms + page_lock_wait_in_ms) > 1000  --1 second
	
		--PRINT 'Starting Step2'
	
		--Step 2: 
		--possible unused or bad indexes.  
		DECLARE dbs CURSOR FOR 
			SELECT name 
			FROM sys.databases
			WHERE name LIKE 'SMS%'
			AND database_id <> db_id()
			AND state_desc = 'ONLINE' 
		   AND user_access_desc = 'MULTI_USER'
		   AND is_in_standby = 0 
		OPEN dbs 
		FETCH NEXT FROM dbs INTO @dbname 
		WHILE @@FETCH_STATUS = 0 
		BEGIN
			--PRINT 'Calling DBA_PM_BadIndexes:' + @dbname
			EXEC DBA_PM_BadIndexes @dbname
			FETCH NEXT FROM dbs INTO @dbname 
		END
		CLOSE dbs
		DEALLOCATE dbs
		
		--Step 3:  
		--Missing Indexes
		INSERT INTO DBA_PM_MissingIndexes
		SELECT TOP 20  
			GETDATE(),
			ROUND(s.avg_total_user_cost * 
					s.avg_user_impact                             
							* (s.user_seeks + s.user_scans),0) 
											AS [Total Cost]
			, d.[statement] AS [Table Name]
			, equality_columns
			, inequality_columns
			, included_columns
		FROM sys.dm_db_missing_index_groups g 
		JOIN sys.dm_db_missing_index_group_stats s 
			ON s.group_handle = g.index_group_handle 
		JOIN sys.dm_db_missing_index_details d 
			ON d.index_handle = g.index_handle
		WHERE d.database_id NOT IN (select database_id from sys.databases WHERE name IN ('master','distribution','msdb'))  
		ORDER BY [Total Cost] DESC   
		
		
	
	END;   

END
GO  