IF exists (select 1 from sysobjects where name='DBA_PM_Addin_PerfCounters' and type='p')
   DROP procedure DBA_PM_Addin_PerfCounters
GO


CREATE PROCEDURE DBA_PM_Addin_PerfCounters
as
BEGIN
SET NOCOUNT ON

DECLARE @Now DATETIME  , @nowprev DATETIME        
SELECT @Now = GETDATE()  
SELECT @nowprev = MAX(Currtime) from dbo.DBA_PM_PerfCounters       

DECLARE @PollingInterval int
SELECT @PollingInterval = COALESCE(ConfigValueInt,1) 
FROM DBA_PM_Config
WHERE ConfigSetting = 'DBA_PM_Addin_PerfCounters'       

--we only want to gather these metrics approx once/hour.  
if datediff (hh,COALESCE(@nowprev,'1/1/1980'),@now) > @PollingInterval                      
BEGIN                      
	--Determine the prefix for the Object_name field
	declare @servicename varchar(200) 
	select @servicename=case when @@servicename<>'mssqlserver' then 'MSSQL$' + @@servicename +':'  else 'sqlserver:' end 
	
	Create table #PerfCounters
	(Currtime datetime 
	,object_name varchar(256)
	,counter_name varchar(256)
	,instance_name varchar(256)
	,cntr_value bigint
	,cntr_type int
	)

	--Perf Counters that can be written as-is
	INSERT INTO DBA_PM_PerfCounters
	  (Currtime, ServerName,object_name, counter_name, cntr_value)
	  SELECT @Now,@@Servername,object_name, counter_name, cntr_value 
	  FROM sys.dm_os_performance_counters
	  where object_name = @servicename+'Memory Manager'  
	  OR (   object_name = @servicename+'SQL Statistics'
		 and counter_name in ('Batch Requests/sec'
							 ,'SQL Compilations/sec'
							 ,'SQL Re-Compilations/sec'
							 )
		 )

	--Perf Counters that need computation
	INSERT INTO #PerfCounters
	SELECT @Now,* 
	  FROM sys.dm_os_performance_counters
	  where object_name = @servicename+'Buffer Manager'
	  and counter_name in (
	  'Checkpoint pages/sec',
	  'Lazy writes/sec',
	  'Page life expectancy',
	  'Page lookups/sec',
	  'Page reads/sec', 
	  'Page writes/sec',
	  'Stolen pages',
	  'Free list stalls/sec'
	  )
		 
	INSERT INTO DBA_PM_PerfCounters
	  (Currtime, ServerName,object_name, counter_name, cntr_value)
	  select @now, @@Servername,t1.object_name, t1.counter_name,
	  CASE 
		WHEN t2.cntr_Value = 0 then t1.cntr_value
		WHEN t2.CurrTime = t1.currtime then 0
		ELSE (t1.cntr_value-t2.cntr_value)/(datediff(ss,t2.CurrTime, t1.currtime))
	  END 
	  from #PerfCounters t1
	  inner join DBA_PM_PerfCountersPrev t2
	  ON t1.Object_name = t2.object_name
	  AND t1.counter_name = t2.counter_name
	  AND isnull(t1.counter_name,'') = isnull(t2.counter_name,'')

	TRUNCATE TABLE DBA_PM_PerfCountersPrev 

	INSERT INTO DBA_PM_PerfCountersPrev 
	  SELECT * FROM  #PerfCounters
	UPDATE DBA_PM_PerfCountersPrev 
	  SET cntr_value=0 where counter_name='Page life expectancy'
	                                                                                           
	DROP TABLE #PerfCounters  

   
END    

END;
GO                                                                                                          
