IF exists (select 1 from sysobjects where name='DBA_PM_Addin_PlanCache' and type='p')
   DROP procedure DBA_PM_Addin_PlanCache
GO

  
CREATE Procedure DBA_PM_Addin_PlanCache       
As    
BEGIN        
	set nocount on             
	
	--these DMVs are not available on 2005
	if dbo.DBA_PM_SQLVersion () = '2005' 
	BEGIN
		RETURN
	END
	    
	DECLARE @Now DATETIME , @nowprev DATETIME     , @dbname varchar(200) 
	SELECT @nowprev = MAX(Currtime) from dbo.DBA_PM_AdHocPlanCacheBloat  
	SELECT @now = GETDATE()    
	
	DECLARE @PollingInterval int
	SELECT @PollingInterval = COALESCE(ConfigValueInt,7) 
	FROM DBA_PM_Config
	WHERE ConfigSetting = 'DBA_PM_Addin_PlanCache'
	
	--let's run this about every 7 hours.  
	if datediff (hh,COALESCE(@nowprev,'1/1/1980'),@now) > @PollingInterval   
	BEGIN	
		--Step 1:
		-- Gives you the text and size of single-use ad-hoc queries  that waste space in the plan cache
		-- Enabling 'optimize for ad hoc workloads' for the instance can help (SQL Server 2008 and 2008 R2 only)
		-- Enabling forced parameterization for the database can help, but test first!
		-- Find single-use, ad-hoc queries that are bloating the plan cache
		INSERT INTO dbo.DBA_PM_AdHocPlanCacheBloat
		SELECT TOP(50) 
			GETDATE()	AS CurrTime,
			@@SERVERNAME AS ServerName,
			[text] AS [QueryText], 
			cp.size_in_bytes
		FROM sys.dm_exec_cached_plans AS cp WITH (NOLOCK)
		CROSS APPLY sys.dm_exec_sql_text(plan_handle) 
		WHERE cp.cacheobjtype = N'Compiled Plan' 
		AND cp.objtype = N'Adhoc' 
		AND cp.usecounts = 1
		ORDER BY cp.size_in_bytes DESC OPTION (RECOMPILE);
	
	
	
		--Step 2: 
		--Plan Cache stats
		DECLARE dbs CURSOR FOR 
			SELECT name 
			FROM sys.databases
			WHERE name LIKE 'SMS%'
			AND database_id <> db_id()
			AND state_desc = 'ONLINE' 
		   AND user_access_desc = 'MULTI_USER'
		   AND is_in_standby = 0 
		OPEN dbs 
		FETCH NEXT FROM dbs INTO @dbname 
		WHILE @@FETCH_STATUS = 0 
		BEGIN
			EXEC DBA_PM_PlanCache @dbname
			FETCH NEXT FROM dbs INTO @dbname 
		END
		CLOSE dbs
		DEALLOCATE dbs
		
		--Step 3:
		--Plan Cache stats by Plan Type
		INSERT INTO dbo.DBA_PM_PlanTypeStats
		SELECT
			GETDATE() AS CurrTime,
			@@SERVERNAME AS ServerName,
			OBJTYPE AS PLAN_TYPE,
			COUNT(*) AS PLAN_NUMBERS,
			(SUM(CAST(SIZE_IN_BYTES AS BIGINT))/1024)/1024 AS SIZE_MB,
			AVG(USECOUNTS) AS USE_COUNT
		FROM SYS.DM_EXEC_CACHED_PLANS
		GROUP BY OBJTYPE

		
		
	
	END;   

END
GO  