IF exists (select 1 from sysobjects where name='DBA_PM_BadIndexes' and type='p')
   DROP procedure DBA_PM_BadIndexes
GO

  
CREATE Procedure DBA_PM_BadIndexes (@dbname varchar(200))       
As    
BEGIN        
	set nocount on    
	
	--this is called from DBA_PM_Addin_IndexMetrics which builds a cursor to pass in the db    
	
	DECLARE @SQL VARCHAR(MAX)
	     
	    
	-- Possible Bad NC Indexes (writes > reads)
	-- Look for indexes with high numbers of writes and zero or very low numbers of reads
	-- Consider your complete workload
	-- Investigate further before dropping an index!
	--
	SELECT @SQL='
	INSERT INTO SMSPHsfu.dbo.DBA_PM_PossibleBadIndexes
	SELECT TOP 100 
		GETDATE() AS CurrTime,
		@@SERVERNAME AS ServerName,
		''' + @dbname + ''' AS DBName,
		OBJECT_NAME(s.[object_id],DB_ID(''' + @dbname + ''')) AS [Table Name], 
		i.name AS [Index Name], 
		i.index_id,
		user_updates AS [Total Writes], 
		user_seeks + user_scans + user_lookups AS [Total Reads],
		user_updates - (user_seeks + user_scans + user_lookups) AS [Difference]
	FROM ' + quotename(@dbname) + '.sys.dm_db_index_usage_stats AS s WITH (NOLOCK)
	INNER JOIN ' + quotename(@dbname) + '.sys.indexes AS i WITH (NOLOCK)
	ON s.[object_id] = i.[object_id]
	AND i.index_id = s.index_id
	WHERE OBJECTPROPERTY(s.[object_id],''IsUserTable'') = 1
	AND s.database_id = DB_ID(''' + @dbname + ''')
	AND user_updates > (user_seeks + user_scans + user_lookups)
	AND i.index_id > 1
	--AND (user_updates - (user_seeks + user_scans + user_lookups)) > 100  --Difference > 100
	ORDER BY [Difference] DESC, [Total Writes] DESC, [Total Reads] ASC 
	OPTION (RECOMPILE);
	'
	EXEC  (@SQL) 

END
GO  