IF exists (select 1 from sysobjects where name='DBA_PM_GatherBufferData' and type='p')
   DROP procedure DBA_PM_GatherBufferData
GO

  
CREATE Procedure DBA_PM_GatherBufferData (@dbname varchar(200))       
As    
BEGIN        
	set nocount on    
	
	--this is called from DBA_PM_Addin_Buffers which builds a cursor to pass in the db   
	 
	--these DMVs are not available on 2005
	if dbo.DBA_PM_SQLVersion () = '2005' 
	BEGIN
		RETURN
	END
	
	DECLARE @SQL VARCHAR(MAX)
	     
	-- Breaks down buffers used by current database by object (table, index) in the buffer cache
	-- Note: This query could take some time on a busy instance
	SELECT @SQL='
	INSERT INTO SMSPHsfu.dbo.DBA_PM_BufferStats
	SELECT TOP 100
		GETDATE() AS CurrTime,
		@@ServerName AS ServerName,
		''' + @dbname + '''AS DBName,
		OBJECT_NAME(p.[object_id],DB_ID(''' + @dbname + ''')) AS [ObjectName], 
		p.index_id, 
		COUNT(*)/128 AS [Buffer size(MB)],  
		COUNT(*) AS [BufferCount], 
		p.data_compression_desc AS [CompressionType], 
		a.type_desc, 
		p.[rows]
	FROM ' + quotename(@dbname) + '.sys.allocation_units AS a WITH (NOLOCK)
	INNER JOIN ' + quotename(@dbname) + '.sys.dm_os_buffer_descriptors AS b WITH (NOLOCK)
		ON a.allocation_unit_id = b.allocation_unit_id
	INNER JOIN ' + quotename(@dbname) + '.sys.partitions AS p WITH (NOLOCK)
		ON a.container_id = p.partition_id
	WHERE b.database_id = CONVERT(int,DB_ID(''' + @dbname + '''))
	AND p.[object_id] > 100
	GROUP BY p.[object_id], p.index_id, p.data_compression_desc, a.type_desc, p.[rows]
	HAVING COUNT(*)/128 > 10  --10MB or larger only
	ORDER BY [BufferCount] DESC OPTION (RECOMPILE);
	'
	exec (@SQL) 

END
GO  