if object_id('DBA_PM_SP_MaintainDeadlockSnapshot') is not null
drop procedure DBA_PM_SP_MaintainDeadlockSnapshot
go


print 'Creating procedure DBA_PM_SP_MaintainDeadlockSnapshot'
go
create procedure DBA_PM_SP_MaintainDeadlockSnapshot
as
begin
  Declare @retentionRows int, @TotalDeadlockRecordCount int
  
  --Futute enhancement: Fish out of a config setting info for retention time/rowcount.
  --For now, retain 1000 deadlocks
  set @retentionRows = 1000

  --Get the deadlocks into a temp table
  
  create table #tmp (DeadlockGraph xml, DeadlockGraphCheckSum int)
  
  insert into #tmp (DeadlockGraph , DeadlockGraphCheckSum)
  select 
  CAST(     --This REPLACE call is to cover a bug in the trace XML from the event system.
			REPLACE(  --The bug is a mismatched begin/end tag pair.
				REPLACE(XEventData.XEvent.value('(data/value)[1]', 'varchar(max)'), 
				'<victim-list>', '<deadlock><victim-list>'),
			'<process-list>','</victim-list><process-list>')
		as xml) as DeadlockGraph
		
  ,checksum(CAST(
			REPLACE(
				REPLACE(XEventData.XEvent.value('(data/value)[1]', 'varchar(max)'), 
				'<victim-list>', '<deadlock><victim-list>'),
			'<process-list>','</victim-list><process-list>')
		as varchar(max))) as DeadlockGraphCheckSum		
  --into #tmp
  FROM
  (select CAST(target_data as xml) as TargetData
   from sys.dm_xe_session_targets st
   join sys.dm_xe_sessions s on s.address = st.event_session_address
   where name = 'system_health'
  ) AS Data
  CROSS APPLY TargetData.nodes ('//RingBufferTarget/event') AS XEventData (XEvent)
  where XEventData.XEvent.value('@name', 'varchar(4000)') = 'xml_deadlock_report'

  insert into DBA_PM_DeadlockSnapshot (DeadlockGraph, DeadlockGraphCheckSum)
  select tmp.DeadlockGraph, tmp.DeadlockGraphCheckSum from #tmp tmp
  left outer join DBA_PM_DeadlockSnapshot DBA 
  on (tmp.DeadlockGraphCheckSum = dba.DeadlockGraphCheckSum and convert(varchar(max),tmp.DeadlockGraph) = convert(varchar(max),dba.DeadlockGraph ))
  where DBA.DeadlockGraphCheckSum is null
  
  --enforce retention
  select @TotalDeadlockRecordCount = max(DeadlockSnapshotId) - min(DeadlockSnapshotId) 
  from DBA_PM_DeadlockSnapshot
  if  @TotalDeadlockRecordCount > @retentionRows
  begin
    delete from DBA_PM_DeadlockSnapshot 
    where DeadlockSnapshotId <= (select max(DeadlockSnapshotId)
                                 from DBA_PM_DeadlockSnapshot) - @retentionRows
  end
end
go
--exec DBA_PM_SP_MaintainDeadlockSnapshot  --print @@servername
--go