IF EXISTS (select 1 from sysobjects where name='DBA_PM_WaitAndBlockInspector' and xtype='P')
   DROP PROCEDURE DBA_PM_WaitAndBlockInspector
GO



CREATE  PROC DBA_PM_WaitAndBlockInspector                
AS                          
BEGIN

SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED                          
SET NOCOUNT ON  


DECLARE	@RiteNow	datetime;
SELECT	@RiteNow	= GETDATE();
	
	WITH BlockAndWaitStats_CTE 
		(CurrTime,spid,BlockingSpid,IsHeadBlocker,waittime,wait_type,wait_resource,resource_description,status,CommandType,DBName,cpu_ms,physical_io,SQLText,program_name,hostname,login_name,last_batch,open_tran,tran_iso_level,blocking_task_address,waiting_task_address,statement_start_offset,statement_end_offset)
	AS (
		SELECT 
		@RiteNow	AS Now,
		p.spid AS spid,
		COALESCE(p.blocked,0) AS BlockingSpid,
		0 AS IsHeadBlocker,
		p.waittime AS waittime,
		substring(p.lastwaittype,1,60) as wait_type,
		substring(p.waitresource,1,512) as wait_resource,
		substring(dowt.resource_description,1,1024) as resource_description,	
		COALESCE(p.status,'sleeping') as Status,										--per BOL
		substring(p.cmd,1,32) AS CommandType,
		substring(db_name(p.dbid),1,50)	AS DBName,
		p.cpu as cpu_ms,
		p.physical_io as physical_io,
		s2.text AS SQLText,
		substring(p.program_name,1,128) as Program_Name,
		substring(p.hostname,1,256) as HostName,
		p.loginame,
		p.last_batch AS last_batch,
		p.open_tran AS open_tran,											
		COALESCE(r.transaction_isolation_level,99) AS tran_iso_level,									--this might be an abandoned spid, we'll decode this further below
		dowt.blocking_task_address as blocking_task_address,
		dowt.waiting_task_address	,
		p.stmt_start,
		p.stmt_end
		FROM master.dbo.sysprocesses p WITH (NOLOCK) 
		LEFT JOIN sys.dm_exec_requests r WITH (NOLOCK)
			ON p.spid = r.session_id	
		LEFT OUTER JOIN sys.dm_os_waiting_tasks dowt with (nolock)
			ON r.session_id = dowt.session_id	
		OUTER APPLY
			(
				 SELECT *
				 FROM sys.dm_exec_sql_text(p.sql_handle)
			) s2
		--WHERE (r.command NOT IN  
		--	('LOG WRITER','LAZY WRITER','LOCK MONITOR','CHECKPOINT SLEEP','TASK MANAGER','SIGNAL HANDLER','TRACE QUEUE TASK',
		--	'BRKR TASK','CHECKPOINT','BRKR EVENT HNDLR','SLEEP_TASK','WAITFOR','RESOURCE MONITOR')
		--	OR r.command IS NULL)  --when r.command IS NULL this equates to sysprocesses.Command = 'AWAITING COMMAND'
		WHERE p.spid <> @@SPID																	--filter out system processes and this spid
		AND p.lastwaittype NOT IN ('SOS_SCHEDULER_YIELD')										--false positives
		AND p.spid > 50
		AND (r.blocking_session_id is not null																							--show spids that are blocked 
		OR r.session_id in (select session_id from sys.dm_os_waiting_tasks WITH (nolock))												--...or are in a waiting state
		OR p.spid in (select distinct blocking_session_id from sys.dm_exec_requests WITH (nolock) WHERE blocking_session_id <> 0)		--...or are causing blocking	
		)
	)
	, HeadBlockers_CTE
		(head_blocker_task_address, head_blocker_session_id)
	AS (
		select 
			blocking_task_address as head_blocker_task_address,
			BlockingSpid as head_blocker_session_id
		from BlockAndWaitStats_CTE
		where blocking_task_address is not null 
		OR BlockingSpid is not null
		EXCEPT
		select 
			waiting_task_address, 
			spid
		from BlockAndWaitStats_CTE
		where blocking_task_address is not null 
		OR BlockingSpid is not null
	)
	--------------------------------------------------
	-- Now post-process the sysprocesses snapshot
	INSERT INTO dbo.DBA_PM_BlockAndWaitStats 
			(CurrTime	
			,spid	
			,BlockingSpid
			,IsHeadBlocker	
			,ServerName	
			,DBName								
			,waittime		
			,wait_type	
			,wait_resource
			,resource_description		
			,status			
			,CommandType						
			,cpu_ms			
			,physical_io									
			,SQLText				
			,program_name	
			,hostname		
			,login_name
			,last_batch	
			,open_tran
			,tran_iso_level
			,statement_start_offset
			,statement_end_offset
			)		
	SELECT 
		CurrTime, 
		p.spid,
		p.BlockingSpid,
		CASE WHEN hb.head_blocker_task_address IS NOT NULL THEN 1 ELSE 0 END AS IsHeadBlocker,
		SUBSTRING(@@SERVERNAME,1,40) AS ServerName,
		p.DBName,
		p.waittime,
		p.wait_type,
		p.wait_resource,
		p.resource_description,
		p.status,
		p.CommandType,
		p.cpu_ms,
		p.physical_io,
		p.SQLText	,
		p.program_name,
		p.hostname,
		p.login_name,
		p.last_batch,
		p.open_tran,
		CASE p.tran_iso_level 
			WHEN 1 THEN 'ReadUncommitted'
			WHEN 2 THEN 'ReadCommitted'
			WHEN 3 THEN 'Repeatable'
			WHEN 4 THEN 'Serializable'
			WHEN 5 THEN 'Snapshot'
			WHEN 99 THEN 'Abandoned Blocking Spid for > 60 sec?'  --this text is referenced in DBA_PM_Addin_AbandonedSpidKiller
			ELSE 'Unspecified'
		END,
		p.statement_start_offset,
		p.statement_end_offset
	FROM BlockAndWaitStats_CTE p
	LEFT JOIN HeadBlockers_CTE hb
		ON p.blocking_task_address = hb.head_blocker_task_address
		AND p.BlockingSpid = hb.head_blocker_session_id
	LEFT JOIN (SELECT DISTINCT BlockingSpid FROM BlockAndWaitStats_CTE WHERE BlockingSpid <> 0) Blockers
		ON p.spid = Blockers.BlockingSpid
	WHERE p.tran_iso_level <> 99				--any non-abandoned spids 
	OR											--abandoned spid logic...
		(									
			p.tran_iso_level = 99							--any spid that appears abandoned or open...
			AND DATEDIFF(s,p.last_batch,p.CurrTime) > 60	--for at least 60 secs...
			AND Blockers.BlockingSpid IS NOT NULL			--and is blocking something else.  
		)
	;

END;  