IF Exists (SELECT 1 FROM dbo.sysobjects where id = object_id('SEDA_RecoveryModelForDB') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
   DROP Procedure SEDA_RecoveryModelForDB
GO

CREATE Procedure SEDA_RecoveryModelForDB (@DBName varchar(100)=NULL)
AS
DECLARE @RecoveryModel varchar(20)
DECLARE @IsRecoverable bit
DECLARE @Description varchar(400)

IF @DBName IS NULL
   SET @DBName=DB_NAME()

IF dbo.SEDA_IsDBInFullRecovery(@DBNAME) = 1
BEGIN
   SELECT @RecoveryModel = 'FULL'
   SELECT @IsRecoverable = 1
   SELECT @Description = 'Database has Recovery Model set to FULL and is fully recoverable.'
END   
ELSE BEGIN
   SELECT @RecoveryModel = CASE recovery_model
                             WHEN 1 then 'FULL'
                             WHEN 2 then 'BULK_LOGGED'
                             WHEN 3 then 'SIMPLE'
                           END
   FROM sys.databases
   WHERE database_id = DB_ID(@DBName)
     
   SELECT @IsRecoverable = 0
   SELECT @Description = 'Database is currently set to '+@RecoveryModel
   IF @RecoveryModel='FULL'
      SELECT @Description = @Description+', however a FULL BACKUP is required for recoverability.  Please perform a FULL Database Backup for database '+@DBName+'.'
   ELSE
      SELECT @Description = @Description+'.  Please change Recovery Model for database '+@DBName+' to FULL and perform a FULL Database Backup.'
END

SELECT @DBName as DatabaseName, @RecoveryModel as CurrentRecoveryModel, @IsRecoverable as IsRecoverable, @Description as Instructions

GO