If exists (select 1 from sysobjects where name='SEDA_SpaceUsedByDatabase' and xtype='p')
   DROP Procedure SEDA_SpaceUsedByDatabase
GO


CREATE PROC SEDA_SpaceUsedByDatabase (@DBName varchar(100)=NULL, @CurrTime datetime=NULL)
AS
SET NOCOUNT ON

IF @CurrTime IS NULL SELECT @CurrTime =getdate()

------------------------------------------------------------
--Identify the Max(currtime) for each DBName
------------------------------------------------------------
select DBName, Max(CurrTime) as CurrTime
into #MaxCurrTimeForDB
from DBA_PM_SpaceUsedByFiles
WHERE (  @DBName IS NULL
      OR DBName = @DBName
      )
group by DBName

------------------------------------------------------------
--For the latest entry, get the Distinct Drives and Freespace
------------------------------------------------------------
select distinct SU.DBName, DriveMP, DriveMPFree
into #DBFreeOnDiskAll
from #MaxCurrTimeForDB t1
inner join DBA_PM_SpaceUsedByFiles SU
on t1.DBName = SU.DBName
and t1.Currtime = SU.CurrTime
WHERE SU.AUTOGROW <> '0 KB'  
group by SU.DBName, DriveMP, DriveMPFree

select DBName, SUM(DriveMPFree) as SpaceFreeOnDisk
into #DBFreeOnDisk
FROM #DBFreeOnDiskAll
GROUP BY DBName
 
------------------------------------------------------------
--For the latest entry, get Files and FreeSpace
------------------------------------------------------------
select SU.DBName, SUM(SpaceFree) as Sum_FreeSpace, SUM(SpaceUsed) as Sum_SpaceUsed, count(*) as NumFiles
into #DBFreeInFiles
from #MaxCurrTimeForDB t1
inner join DBA_PM_SpaceUsedByFiles SU
on t1.DBName = SU.DBName
and t1.Currtime = SU.CurrTime
group by SU.DBName


select DBName, Max(CurrTime) as CurrTime
into #MaxDT_SpaceUsedByDatabase
from DBA_PM_SpaceUsedByDatabase
group by DBName

Select SD.DBName, SD.CurrTime, SD.SpaceUsed
into #PrevSpaceUsed
from #MaxDT_SpaceUsedByDatabase t1
INNER JOIN DBA_PM_SpaceUsedByDatabase SD
ON t1.DBName = SD.DBName
AND t1.CurrTime = SD.CurrTime


INSERT INTO DBA_PM_SpaceUsedByDatabase
  ( CurrTime, DBName, SpaceUsed, SpaceFreeInFiles, SpaceFreeOnDisk
  , WeeklygrowthRate, ServerName)
  SELECT @CurrTime, FD.DBName, FF.Sum_SpaceUsed, FF.Sum_FreeSpace, FD.SpaceFreeOnDisk
  , CASE 
      WHEN DATEDIFF(hh,P.Currtime,@CurrTime) = 0 THEN 0
      ELSE ((FF.Sum_SpaceUsed-P.SpaceUsed)*168)/DATEDIFF(hh,P.Currtime,@CurrTime)
    END,
    @@SERVERNAME
  FROM #DBFreeOnDisk FD
  INNER JOIN #DBFreeInFiles FF
  ON FD.DBName = FF.DBName
  LEFT OUTER JOIN #PrevSpaceUsed P
  ON FD.DBName = P.DBName

GO